/*  cpuctrl for GP2X
    Copyright (C) 2005  Hermes/PS2Reality 
	the gamma-routine was provided by theoddbot
	parts (c) Rlyehs Work & (C) 2006 god_at_hell 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifdef CAANOO
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/ioctl.h>

enum _pollux_clock_ioctl {
    IOCTL_SET_PLL0,
    IOCTL_SET_PLL1,
    IOCTL_GET_PLL0,
    IOCTL_GET_PLL1,
    IOCTL_SET_CLK_DEFUALTS
};

enum _pollux_pll0_index {
    PLL0_200 = 0,
    PLL0_300,
    PLL0_400,
	PLL0_470,
	PLL0_530,
	PLL0_600,
	PLL0_620,
	PLL0_650,
	PLL0_680,
	PLL0_710,
	PLL0_750,
	PLL0_780,
	PLL0_800,
	PLL0_830,
	PLL0_850,
	PLL0_870,
	PLL0_900
};    

enum _pollux_pll1_index {
    PLL1_261 = 0,
    PLL1_132_75,
    PLL1_124_2	
};

unsigned int caanoo_cpuclock[17] = {200, 300, 400, 470, 530, 600, 620, 650, 680, 710, 750, 780, 800, 830, 850, 870, 900};


int SetCaanooClock(int step)
{
	int fd=-1;
	
	fd=open("/dev/pollux_clock" , O_RDWR);
	if(fd>=0)
	{
		if(step==-1)
		{
			if(ioctl(fd, IOCTL_SET_CLK_DEFUALTS, NULL)<0)
				fprintf(stderr,"Failed to change CPU clock to default\n");
			else 
				fprintf(stderr,"CPU clock has changed to default\n");
		}
		else if(step<=10)
		{
			if(ioctl(fd, IOCTL_SET_PLL0, &step)<0)
				fprintf(stderr,"Failed to change CPU clock to %d MHz\n",caanoo_cpuclock[step]);
			else 
				fprintf(stderr,"CPU clock has changed to %d\n",caanoo_cpuclock[step]);
		}
		else
			fprintf(stderr,"Invalid clock value\n");

		usleep(200000);
		close(fd);
		return 0;
	}

	fprintf(stderr,"Failed to open clock device\n");
	return -1;
}
#else
#include <fcntl.h>
#include <sys/mman.h>
#include <stdio.h>

#define SYS_CLK_FREQ 7372800
volatile unsigned short *MEM_REG;
unsigned long   gp2x_dev;
unsigned short *gp2x_memregs;

void Set920Clock(int MHZ)
{
	printf("Set 920 clock at %d\n", MHZ);
	
	if(!gp2x_dev) gp2x_dev = open("/dev/mem", O_RDWR);
	gp2x_memregs=(unsigned short *)mmap(0, 0x10000, PROT_READ|PROT_WRITE, MAP_SHARED, gp2x_dev, 0xc0000000);

	// Init CPU control
	MEM_REG = &gp2x_memregs[0];

	// Set 920t clock
	unsigned v;
	unsigned mdiv,pdiv=3,scale=0;
	MHZ *= 1000000;
	mdiv = (MHZ*pdiv) / SYS_CLK_FREQ;
	mdiv = ((mdiv-8)<<8) & 0xff00;
	pdiv = ((pdiv-2)<<2) & 0xfc;
	scale &= 3;
	v = mdiv | pdiv | scale;
	MEM_REG[0x910>>1] = v;
	
	while(MEM_REG[0x0902>>1] & 1);    //Wait for the frequentie to be ajused
}

// normal =	set_RAM_Timings(8, 16, 3, 8, 8, 8, 8);
void set_RAM_Timings(int tRC, int tRAS, int tWR, int tMRD, int tRFC, int tRP, int tRCD)
{
	printf("Set RAM timing:\n");
	printf("tRC=%d tRAS=%d tWR=%d tMRD=%d tRFC=%d tRP=%d tRCD=%d\n",tRC, tRAS, tWR, tMRD, tRFC, tRP, tRCD);
	tRC -= 1; tRAS -= 1; tWR -= 1; tMRD -= 1; tRFC -= 1; tRP -= 1; tRCD -= 1; // ???
	MEM_REG[0x3802>>1] = ((tMRD & 0xF) << 12) | ((tRFC & 0xF) << 8) | ((tRP & 0xF) << 4) | (tRCD & 0xF);
	MEM_REG[0x3804>>1] = /*0x9000 |*/ ((tRC & 0xF) << 8) | ((tRAS & 0xF) << 4) | (tWR & 0xF);
}
#endif
